#!/bin/bash

# global vars
. ~/.profile
VARIABLES=`echo $0|sed 's/switch_server.sh/server.config/'`
. $VARIABLES

# local vars
LOGGING=1
LOGFILE=$LOGPATH/switch.log
DELAYTIME=`echo $DATETIME+30|bc`

if test "$LOGGING" -eq 1; then
	echo "" >> $LOGFILE
	echo "nagios-serverswitch script started `date`" >> $LOGFILE
fi

# check parameters
if test -z "$1" -o -z "$2" -o -z "$2" -o -z "$3" -o -z "$4" -o -z "$5"; then
	if test "$LOGGING" -eq 1; then
		echo "" >> $LOGFILE
		echo ">> missing parameters! end." >> $LOGFILE
		echo "" >> $LOGFILE
	fi
	exit 0;
fi

# functions
start_server()
{
	if test "$LOGGING" -eq 1; then
		echo "start_server:" >> $LOGFILE
	fi

	##################
	# stop the redundancy-server
	CMDLINE="[$DATETIME] SHUTDOWN_PROGRAM;$DATETIME"
       	echo $CMDLINE >> $COMMANDFILERED
	if test "$LOGGING" -eq 1; then
		echo $CMDLINE >> $LOGFILE
	fi

	##################
	# copy actual config- and rrd-files (directories "etc" and "rrd")
	if test "$LOGGING" -eq 1; then
		echo "start_server:" >> $LOGFILE

		# get actual configuration and rrd-files
		echo "copying configfiles and actual data from other server..." >> $LOGFILE
		echo "copying: `scp -r ${NAGIOSUSER}@${OTHERSERVER}:${NAGIOSPATH}/APAN/* ${NAGIOSPATH}/APAN/`" >> $LOGFILE
		echo "copying: `scp -r ${NAGIOSUSER}@${OTHERSERVER}:${NAGIOSPATH}/etc/* ${NAGIOSPATH}/etc/`" >> $LOGFILE
		echo "copying: `scp -r ${NAGIOSUSER}@${OTHERSERVER}:${NAGIOSPATH}/rrd/* ${NAGIOSPATH}/rrd/`" >> $LOGFILE
		echo "copying: `scp -r ${NAGIOSUSER}@${OTHERSERVER}:${NAGIOSPATH}/eventhandler/* ${NAGIOSPATH}/eventhandler/`" >> $LOGFILE
	fi

	##################
	# start the apache server
	WRITE=`${APACHEPATH}/bin/apachectl start` 
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi

	##################
	# starting nagios
	# remove old 'nagios.cmd' and 'nagios.lock' file (if theres one)
	if test "$LOGGING" -eq 1; then
		echo "removing nagios.cmd..." >> $LOGFILE
	fi
	WRITE=`rm ${NAGIOSPATH}/var/rw/nagios.cmd` 
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi
	if test "$LOGGING" -eq 1; then
		echo "removing nagios.lock..." >> $LOGFILE
	fi
	WRITE=`rm ${NAGIOSPATH}/var/nagios.lock` 
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi

	if test "$LOGGING" -eq 1; then
		echo "executing:" >> $LOGFILE
		echo "${NAGIOSPATH}/bin/nagios -d ${NAGIOSPATH}/etc/nagios.cfg" >> $LOGFILE
	fi
	WRITE=`${NAGIOSPATH}/bin/nagios -d ${NAGIOSPATH}/etc/nagios.cfg`
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi

	##################
	# the backupserver needs to acknowledge the FAME-switch-process
	if test "$SELFHOST" = "$BACKUPSERVER"; then
		# wait a few secs or use DELAYTIME for acknowledge
		sleep 60

		# DISABLE_SVC_NOTIFICATIONS for SWITCH-SERVICE-CHECK
		CMDLINE="[$DATETIME] DISABLE_SVC_NOTIFICATIONS;${MAINSERVER};${SERVICEDESC}"
       		echo $CMDLINE >> $COMMANDFILE
		if test "$LOGGING" -eq 1; then
			echo $CMDLINE >> $LOGFILE
		fi

		# ACKNOWLEDGE_SVC_PROBLEM for SWITCH-SERVICE-CHECK
		CMDLINE="[$DELAYTIME] ACKNOWLEDGE_SVC_PROBLEM;${MAINSERVER};${SERVICEDESC};1;automatic acknowledge by switch-script"
       		echo $CMDLINE >> $COMMANDFILE
		if test "$LOGGING" -eq 1; then
			echo $CMDLINE >> $LOGFILE
		fi
	fi
}

stop_server()
{
	if test "$LOGGING" -eq 1; then
		echo "stop_server:" >> $LOGFILE
	fi

	##################
        # shutdown server
	if test "$LOGGING" -eq 1; then
		echo "shuting down local nagiosserver" >> $LOGFILE
	fi
        CMDLINE="[$DATETIME] SHUTDOWN_PROGRAM;${DATETIME}"
        echo $CMDLINE >> $COMMANDFILE
	if test "$LOGGING" -eq 1; then
		echo $CMDLINE >> $LOGFILE
	fi

	##################	
	# stop the apache-server
	if test "$LOGGING" -eq 1; then
		echo "shutting down the apache server..." >> $LOGFILE
	fi
	WRITE=`${APACHEPATH}/bin/apachectl stop`
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi

	##################	
	# ...then start the other server
	if test "$LOGGING" -eq 1; then
		echo "starting nagiosserver on ${OTHERSERVER} via ssh" >> $LOGFILE
	fi
	WRITE=`ssh ${NAGIOSUSER}@${OTHERSERVER} "LD_LIBRARY_PATH=$NAGIOSHOME/lib ${NAGIOSPATH}/eventhandler/switch_server.sh $SERVICEDESC $SERVICESTATE $STATETYPE $SERVICEATT $SELFHOST"`
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi
	
	##################	
	# start the redundancy-server
	# remove old 'nagios.cmd' and 'nagios.lock' file (if theres one)
	if test "$LOGGING" -eq 1; then
		echo "removing nagios.cmd..." >> $LOGFILE
	fi
	WRITE=`rm ${NAGIOSREDPATH}/var/rw/nagios.cmd` 
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi
	if test "$LOGGING" -eq 1; then
		echo "removing nagios.lock..." >> $LOGFILE
	fi
	WRITE=`rm ${NAGIOSREDPATH}/var/nagios.lock` 
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi

	if test "$LOGGING" -eq 1; then
		echo "starting redundancy-nagios-server..." >> $LOGFILE
	fi
	WRITE=`$NAGIOSREDPATH/bin/nagios -d $NAGIOSREDPATH/etc/nagios.cfg`
	if test "$LOGGING" -eq 1; then
		echo $WRITE >> $LOGFILE
	fi
}


##################	
# main script #
##################	

# take the parameters
echo "Parameters:" >> $LOGFILE
SERVICEDESC=$1
SERVICESTATE=$2
STATETYPE=$3
SERVICEATT=$4
EXECUTER=$5

if test "$LOGGING" -eq 1; then
	echo "SERVICEDESC : $1" >> $LOGFILE
	echo "SERVICESTATE: $2" >> $LOGFILE
	echo "STATETYPE   : $3" >> $LOGFILE
	echo "SERVICEATT. : $4" >> $LOGFILE
	echo "Executed by : $5" >> $LOGFILE
fi

if test "$SERVICEATT" -eq 2; then
	##################
	# main-server:
	if test "$SELFHOST" = "$MAINSERVER"; then
		if test "$SERVICESTATE" = "OK" -a "$SELFHOST" != "$EXECUTER"; then
			if test "$LOGGING" -eq 1; then
				echo "MAINNAGIOSSERVER RECOVERING!" >> $LOGFILE
			fi
			start_server
		fi
		if test "$SERVICESTATE" = "CRITICAL"; then
			if test "$LOGGING" -eq 1; then
				echo "MAINNAGIOSSERVER SHUTTING DOWN!" >> $LOGFILE
			fi
			stop_server
		fi	
	fi

	##################
	# backup-server:
	if test "$SELFHOST" = "$BACKUPSERVER"; then
		if test "$2" = "CRITICAL" -a "$SELFHOST" != "$EXECUTER"; then
			if test "$LOGGING" -eq 1; then
				echo "BACKUPNAGIOSSERVER RECOVERING!" >> $LOGFILE
			fi
			start_server
		fi
		if test "$SERVICESTATE" = "OK"; then
			if test "$LOGGING" -eq 1; then
				echo "BACKUPNAGIOSSERVER SHUTING DOWN!" >> $LOGFILE       
			fi
			stop_server
		fi
	fi
else
	if test "$LOGGING" -eq 1; then
		echo "Serviceattempt no. ${SERVICEATT}; did nothing" >> $LOGFILE
	fi
fi

if test "$LOGGING" -eq 1; then
	echo "ready." >> $LOGFILE
	echo "" >> $LOGFILE
fi
exit

